local VERSION = tonumber(("$Revision: 57478 $"):match("%d+"))

PartyToGo = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "Sink-1.0")

function PartyToGo:OnInitialize()
	self:RegisterDB("PartyToGoDB")
	self:RegisterDefaults("profile", {
		announce = {
			complete = false,
			objective = false,
			objectiveprogress = false,
		},
		notify = {
			complete = true,
			objective = true,
			objectiveprogress = true,
		},
		colors = {
			notstarted = {1,0,0},
			underway = {1,1,0},
			done = {0,1,0},
		},
	})
	local colorget = function(k)
		return unpack(self.db.profile.colors[k])
	end
	local colorset = function(k, r, g, b, a)
		local t = self.db.profile.colors[k]
		t[1] = r
		t[2] = g
		t[3] = b
		t[4] = a
	end
	self:RegisterChatCommand({"/p2g", "/partytogo",}, {
		name = "PartyToGo",
		desc = "Keep you up to date on your party's status, and them up to date on yours",
		type = "group",
		args = {
			announce = {
				name = "Announce",
				desc = "Announce quest status in party chat",
				type = "group",
				args = {
					complete = {
						type = "toggle", name = "Quest Completion",
						desc = "Notify of quest completion",
						get = function() return self.db.profile.announce.complete end,
						set = function(t) self.db.profile.announce.complete = t end,
					},
					objective = {
						type = "toggle", name = "Objective Completion",
						desc = "Notify of individual objective completion",
						get = function() return self.db.profile.announce.objective end,
						set = function(t) self.db.profile.announce.objective = t end,
					},
					progress = {
						type = "toggle", name = "Objective Progress",
						desc = "Notify of any progress on an objective",
						get = function() return self.db.profile.announce.objectiveprogress end,
						set = function(t) self.db.profile.announce.objectiveprogress = t end,
					},
				},
			},
			notify = {
				name = "Notify",
				desc = "Local announcements when people in your party complete an objective",
				type = "group",
				args = {
					complete = {
						type = "toggle", name = "Quest Completion",
						desc = "Notify of quest completion",
						get = function() return self.db.profile.notify.complete end,
						set = function(t) self.db.profile.notify.complete = t end,
					},
					objective = {
						type = "toggle", name = "Objective Completion",
						desc = "Notify of individual objective completion",
						get = function() return self.db.profile.notify.objective end,
						set = function(t) self.db.profile.notify.objective = t end,
					},
					progress = {
						type = "toggle", name = "Objective Progress",
						desc = "Notify of any progress on an objective",
						get = function() return self.db.profile.notify.objectiveprogress end,
						set = function(t) self.db.profile.notify.objectiveprogress = t end,
					},
				},
			},
			colors = {
				type = 'group', name = "Colors",
				desc = "Choose custom colors",
				order = 99,
				args = {
					completion = {type = 'header', name = "Completion", order = 7,},
					notstarted = {type = 'color', name = "Not started", desc = "Not started", hasAlpha = false, order = 8, passValue="notstarted", get=colorget, set=colorset},
					underway = {type = 'color', name = "Underway", desc = "Underway", hasAlpha = false, order = 10, passValue="underway", get=colorget, set=colorset},
					done = {type = 'color', name = "Done", desc = "Done", hasAlpha = false, order = 12, passValue="done", get=colorget, set=colorset},
				},
			},
			output = AceLibrary("Sink-1.0"):GetAceOptionsDataTable(self).output,
		},
	})
end

function PartyToGo:OnEnable()
	self:RegisterEvent("Quixote_Party_Leaderboard_Update")
	self:RegisterEvent("Quixote_Party_Quest_Complete")
	self:RegisterEvent("Quixote_Leaderboard_Update")
	self:RegisterEvent("Quixote_Quest_Complete")
end

function PartyToGo:Quixote_Party_Leaderboard_Update(sender, title, description, numHad, numGot, numNeeded)
	local db = self.db.profile.notify
	if numGot == numNeeded then
		if db.objective or db.objectiveprogress then
			self:Pour(ERR_QUEST_COMPLETE_S:format("("..sender..":"..title..") "..description), self:GetColorFromCompletion(1))
		end
	elseif db.objectiveprogress then
		self:Pour(ERR_QUEST_ADD_ITEM_SII:format("("..sender..":"..title..") "..description, numGot, numNeeded), self:GetColorFromCompletion(tonumber(numGot)/tonumber(numNeeded)))
	end
end

function PartyToGo:Quixote_Leaderboard_Update(title, id, lid, description, numHad, numGot, numNeeded)
	local db = self.db.profile.announce
	if (db.objective or db.objectiveprogress) and GetNumPartyMembers() > 0 then
		if numGot == numNeeded then
			if db.objective or db.objectiveprogress then
				SendChatMessage(ERR_QUEST_COMPLETE_S:format("("..title..") "..description), "PARTY")
			end
		elseif db.objectiveprogress then
			SendChatMessage(ERR_QUEST_ADD_ITEM_SII:format("("..title..") "..description, numGot, numNeeded), "PARTY")
		end
	end
end

function PartyToGo:Quixote_Party_Quest_Complete(sender, title)
	if self.db.profile.notify.questcomplete then
		self:Pour(ERR_QUEST_COMPLETE_S:format("("..sender..") "..title), self:GetColorFromCompletion(1))
	end
end

function PartyToGo:Quixote_Quest_Complete(title, id)
	if self.db.profile.announce.complete and GetNumPartyMembers() > 0 then
		SendChatMessage(ERR_QUEST_COMPLETE_S:format(title), "PARTY")
	end
end

function PartyToGo:GetColorFromCompletion(percent)
	local colors = self.db.profile.colors
	if percent <= 0 then
		return unpack(colors.notstarted)
	elseif percent >= 1 then
		return unpack(colors.done)
	elseif percent == 0.5 then
		return unpack(colors.underway)
	elseif percent < 0.5 then
		percent = percent / 0.5
		local r1,g1,b1 = unpack(colors.notstarted)
		local r2,g2,b2 = unpack(colors.underway)
		return r1+(r2-r1)*percent, g1+(g2-g1)*percent, b1+(b2-b1)*percent
	elseif percent < 1 then
		percent = (percent-0.5) / 0.5
		local r1,g1,b1 = unpack(colors.underway)
		local r2,g2,b2 = unpack(colors.done)
		return r1+(r2-r1)*percent, g1+(g2-g1)*percent, b1+(b2-b1)*percent
	end
end
